// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "SetDataRefTask.h"

#include "Protocol.h"

//------------------------------------------------------------------------------

using xplra::SetDataRefTask;
using xplra::SetIntDataRefTask;
using xplra::SetFloatDataRefTask;
using xplra::SetDoubleDataRefTask;

using xplcommon::DataStream;

using std::string;

//------------------------------------------------------------------------------

SetDataRefTask* SetDataRefTask::create(uint8_t& result, DataStream& stream)
{
    result = Protocol::RESULT_OK;

    string name = stream.readString();
    uint8_t type = stream.readU8();
    if (!stream) return 0;

    if (type==Protocol::TYPE_INT) {
        int value = stream.readS32();
        if (stream) return new SetIntDataRefTask(name, value);
    } else if (type==Protocol::TYPE_FLOAT) {
        float value = stream.readFloat();
        if (stream) return new SetFloatDataRefTask(name, value);
    } else if (type==Protocol::TYPE_DOUBLE) {
        double value = stream.readDouble();
        if (stream) return new SetDoubleDataRefTask(name, value);
    } else if (type==Protocol::TYPE_FLOAT_ARRAY ||
               type==Protocol::TYPE_INT_ARRAY ||
               type==Protocol::TYPE_BYTE_ARRAY)
    {
        int length = stream.readS32();
        int offset = stream.readS32();
        if (!stream) return 0;
        if (length<=0 || length>Protocol::MAX_LENGTH) {
            result = Protocol::RESULT_INVALID_LENGTH;
            return 0;
        }
        if (offset<0) {
            result = Protocol::RESULT_INVALID_OFFSET;
            return 0;
        }

        SetDataRefTask* task = 0;
        if (type==Protocol::TYPE_INT_ARRAY) {
            task = new SetIntArrayDataRefTask(name, length, offset);
        } else if (type==Protocol::TYPE_FLOAT_ARRAY) {
            task = new SetFloatArrayDataRefTask(name, length, offset);
        } else {
            task = new SetByteArrayDataRefTask(name, length, offset);
        }

        task->readValue(stream);

        if (stream) {
            return task;
        } else {
            delete task;
        }
    } else {
        result = Protocol::RESULT_INVALID_TYPE;
    }

    return 0;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

void SetIntDataRefTask::readValue(DataStream& stream)
{
    value = stream.readS32();
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

void SetFloatDataRefTask::readValue(DataStream& stream)
{
    value = stream.readFloat();
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

void SetDoubleDataRefTask::readValue(DataStream& stream)
{
    value = stream.readDouble();
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
